package lmdb
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
Module Lmdb.Env
Collection of maps stored in a single memory-mapped file.
val create :
_ perm ->
?max_readers:int ->
?map_size:int ->
?max_maps:int ->
?flags:Flags.t ->
?mode:int ->
string ->
tcreate perm path creates an environment with Ro or Rw permissions with data and lock files in the already existing directory path. If no separate directory is desired, Flags.no_subdir can be passed.
The returned handle is not garbage collected and should be closed explicitely to free locks and prevent corruption on async environments.
val sync : ?force:bool -> t -> unitval close : t -> unitval copy : ?compact:bool -> t -> string -> unitval copyfd : ?compact:bool -> t -> Unix.file_descr -> unitval set_flags : t -> Flags.t -> bool -> unitval flags : t -> Flags.tval set_map_size : t -> int -> unitval path : t -> stringval fd : t -> Unix.file_descrval stat : t -> Mdb.statval info : t -> Mdb.envinfoval max_readers : t -> intval max_keysize : t -> intval reader_list : t -> string listval reader_check : t -> int